<?php

use Illuminate\Console\Command;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputArgument;
use Ratchet\Server\IoServer;
use Ratchet\Http\HttpServer;
use FutureHome\Socket\FHServer;
use FutureHome\Socket\SocketServer;

class StartServer extends Command {

	/**
	 * The console command name.
	 *
	 * @var string
	 */
	protected $name = 'start:server';

	/**
	 * The console command description.
	 *
	 * @var string
	 */
	protected $description = 'Starts websocket server that listens to the user request.';

	/**
	 * Create a new command instance.
	 *
	 * @return void
	 */
	public function __construct()
	{
		parent::__construct();
	}

	/**
	 * Execute the console command.
	 *
	 * @return mixed
	 */
	public function fire()
	{
		// initialize logger
        \FutureHome\Socket\Logger::initLogger();
        
		 // Prepare socket server
		$socket_server = new SocketServer();
        $server = IoServer::factory(new HttpServer(new FHServer($socket_server)), Config::get('app.websocket_port'), '127.0.0.1');
        $this->info('Listening on port ' . Config::get('app.websocket_port'));
        $socket_server->setLoop($server->loop);

        // Run socket server
        $server->run();
	}
}
