<?php

namespace FutureHome\Handler;

use Evenement\EventEmitter;
use FutureHome\Socket\Logger;

class Update extends EventEmitter
{
    public $updates = 0;
    public $escapepod_update = 0;

    protected $loop;

    public function __construct($loop)
    {
        $this->loop = $loop;
        $loop->addPeriodicTimer(\Config::get('app.update_check_time'), array($this, 'checkForUpdates'));
    }

    public function checkForUpdates()
    {
        Logger::info('Checking for updates.');
        $updatepath = \Config::get('app.update_file_path');

        $filesystem = new \Illuminate\Filesystem\Filesystem();
        if ($filesystem->exists($updatepath)) {
            $this->updates = (int) $filesystem->get($updatepath);
        }

        $escapeupdatepath = \Config::get('app.escape_pod_update_file_path');
        if ($filesystem->exists($escapeupdatepath)) {
            $this->escapepod_update = (int) $filesystem->get($escapeupdatepath);
        }

        $this->emit('update', array($this->updates, $this->escapepod_update));
    }

    public function runUpdateDownloader()
    {
        $update_download_command = \Config::get('app.update_download_command');
        $process = new \React\ChildProcess\Process($update_download_command);

        $process->on('exit', function ($exit_code, $term_signal) {
            if ($exit_code == 0) {
                $this->emit('download', array('downloaded'));
            } else {
                $this->emit('download', array('download failed'));
            }
        });
        $process->start($this->loop);

        $process->stdout->on('data', function ($output) {
            $this->emit('message', array($output));
            $this->emit('download', array('downloading'));
        });
    }

    public function runUpdate()
    {
        if ($this->updates == 0 && $this->escapepod_update == 0) {
            $this->checkForUpdates();
        }
        if ($this->updates != 0) {
            $update_command = \Config::get('app.update_command');
            $process = new \React\ChildProcess\Process($update_command);

            $process->on('exit', function ($exit_code, $term_signal) {
                if ($exit_code == 0) {
                    $this->emit('updating', array('updated successfully'));
                } else {
                    $this->emit('updating', array('update failed'));
                }
            });
            $process->start($this->loop);

            $process->stdout->on('data', function ($output) {
                $this->emit('message', array($output));
                $this->emit('updating', array('updating'));
            });
        }

        if ($this->escapepod_update != 0) {
            $update_command = \Config::get('app.escape_pod_update_command');
            exec($update_command.' > /dev/null 2>/dev/null & echo $!');
        }
    }
}
