<?php
namespace FutureHome\Socket\Client;

use FutureHome\Socket\Logger;
use Ratchet\ConnectionInterface;
use FutureHome\WSClient\Connector;
use React\EventLoop\LoopInterface;
use React\Dns\Resolver\Resolver;
use Evenement\EventEmitterInterface;
use Evenement\EventEmitterTrait;
use FutureHome\WSClient\WebSocket;

class ClientVinculum extends Connector implements EventEmitterInterface {
	use EventEmitterTrait;

    protected $connected = false;
    protected $connection;

	public function __construct(LoopInterface $loop, Resolver $resolver = null, array $secureContext = []) {
		parent::__construct($loop, $resolver, $secureContext);
	}

	public function isConnected() {
        return $this->connected;
    }

    public function disconnect() {
        if ($this->connection) {
            $this->connection->close();
        }
        $this->connected = false;
    }

	public function connect() {
        $this(sprintf("ws://%s:%s%s", \Config::get('app.vinculum_host'), \Config::get('app.vinculum_port'), '/'), [], [], $this->loop)->then(function(WebSocket $conn) {
            Logger::info('Connected to vinculum.');
            $this->connected = true;
            $this->connection = $conn;
            $this->emit('connected');

            $conn->on('message', [$this, 'onMessage']);
            $conn->on('close', function() {
                $this->emit('close');
            });
        }, function(\Exception $e) {
            $this->emit('error', [$e]);
            Logger::error($e->getMessage());
            Logger::error($e->getTraceAsString());
        });
    }

    public function send($message) {
        $this->connection->send($message);
    }

    public function onMessage($msg, $from) {
        $this->emit('data', array($from, $msg->getPayload()));
    }
}
