<?php
	namespace FutureHome\Socket\Mapper;

	use Illuminate\Support\SerializableClosure;

	class RequestMap {
		const REQUEST_PREFIX = 'request:';

		protected $_requestMAP;

		public function getRequest($requestId, $connectionHash) {
			return json_decode(\RedisL4::get(self::REQUEST_PREFIX . $connectionHash . ':' . $requestId), true);
		}

		public function addRequest($requestId, $connectionHash, $data = null) {
			if($data == null) {
				$data = $this->getRequest($requestId, $connectionHash);
			}
			// If it is still null
			if($data == null) {
				$data = array('callback' => null);
			}
			\RedisL4::setex(self::REQUEST_PREFIX . $connectionHash . ':' . $requestId, 1*60, json_encode($data));
		}

		public function addCallback($requestId, $connectionHash, $callback) {
			$request = $this->getRequest($requestId, $connectionHash);

			if(!is_array($request)) {
				$request = array();
			}
			else {
				if(!array_key_exists('callback', $request)) {
					$request['callback'] = array();
				}
			}
			$request['callback'][] = serialize(new SerializableClosure($callback));

			$this->addRequest($requestId, $connectionHash, $request);
		}

		public function getCallback() {
			$request = $this->getRequest($requestId, $connectionHash);

			if(array_key_exists('callback', $request)) {
				return $request['callback'];
			}
			return null;
		}
	}