<?php

/**
 * User: arunpoudel
 * Date: 10/24/14
 * Time: 5:25 PM.
 */
namespace FutureHome\Socket\Message\SevenOfNine;

use FutureHome\Socket\Logger;

class Message
{
    const TYPE_NOTIFY = 'notify';
    const TYPE_REQUEST = 'request';
    const TYPE_RESPONSE = 'response';
    const TYPE_PING = 'ping';

    const COMMAND_ADD = 'add';
    const COMMAND_EDIT = 'edit';
    const COMMAND_DELETE = 'delete';
    const COMMAND_GET = 'get';
    const COMMAND_LRN = 'lrn';
    const COMMAND_SET = 'set';
    const COMMAND_UNLRN = 'unlrn';
    const COMMAND_UPDATE = 'update';

    const COMPONENT_DEVICE = 'device';
    const COMPONENT_ROOM = 'room';
    const COMPONENT_MODE = 'mode';
    const COMPONENT_TIMER = 'timer';
    const COMPONENT_CUBE = 'cube';
    const COMPONENT_UPDATE = 'update';
    const COMPONENT_HOUSE = 'house';
    const COMPONENT_USER = 'user';
    const COMPONENT_LOG = 'log';
    const COMPONENT_CONFIG = 'config';
    const COMPONENT_SHORTCUT = 'shortcut';
    const COMPONENT_DEBUG = 'debug';
    const COMPONENT_ALL = 'all';

    const DST_HIVE = 'hive';
    const DST_CUBE = 'cube';
    const DST_VINCULUM = 'vinculum';
    const DST_HYPERCHANNEL = 'hyperchannel';
    const DST_UPDATE = 'update';
    const DST_CLIENT = 'clients';

    const SRC_HIVE = 'hive';
    const SRC_CUBE = 'cube';
    const SRC_VINCULUM = 'vinculum';
    const SRC_HYPERCHANNEL = 'hyperchannel';
    const SRC_UPDATE = 'update';

    const PARAM_PARAM = 'param';
    const PARAM_COUNT = 'count';
    const PARAM_DEVICE = 'device';
    const PARAM_ROOM = 'room';
    const PARAM_USER = 'users';
    const PARAM_MODE = 'mode';
    const PARAM_HOUSE = 'house';
    const PARAM_TIMER = 'timer';
    const PARAM_SHORTCUT = 'shortcut';
    const PARAM_CLIENT = 'client';
    const PARAM_PAGE = 'page';
    const PARAM_LOG = 'log';

    const PARAM_PARAM_SECURITY = 'security';
    const PARAM_PARAM_HEATING = 'heating';

    const PARAM_PARAM_SECURITY_MAGNET = 'magnet';
    const PARAM_PARAM_SECURITY_SMOKE = 'smoke';
    const PARAM_PARAM_SECURITY_PRESENCE = 'presence';

    const PARAM_PARAM_SECURITY_MAGNET_OPEN = 'open';

    const PARAM_PARAM_HEATING_TEMPERATURE = 'temperature';

    const PARAM_HOUSE_MODE = 'mode';

    const PARAM_USER_EMAIL = 'email';
    const PARAM_USER_ROOM = 'room';
    const PARAM_USER_SETTING = 'setting';
    const PARAM_USER_MODE = 'mode';

    const CLIENT_NAME = 'name';

    protected static $version = 'sevenOfNine';

    public static $valid_types = array(
        self::TYPE_REQUEST,
        self::TYPE_RESPONSE,
        self::TYPE_NOTIFY,
        self::TYPE_PING,
    );

    protected static $valid_commands = array(
        self::COMMAND_ADD,
        self::COMMAND_EDIT,
        self::COMMAND_DELETE,
        self::COMMAND_GET,
        self::COMMAND_LRN,
        self::COMMAND_SET,
        self::COMMAND_UNLRN,
        self::COMMAND_UPDATE,
    );

    protected static $valid_components = array(
        self::COMPONENT_DEVICE,
        self::COMPONENT_ROOM,
        self::COMPONENT_MODE,
        self::COMPONENT_TIMER,
        self::COMPONENT_CUBE,
        self::COMPONENT_UPDATE,
        self::COMPONENT_HOUSE,
        self::COMPONENT_USER,
        self::COMPONENT_LOG,
        self::COMPONENT_CONFIG,
        self::COMPONENT_SHORTCUT,
        self::COMPONENT_ALL,
    );

    protected $type;

    protected $src;

    protected $dst;

    protected $data;

    public function __construct(\stdClass $message)
    {
        if (isset($message->msg)) {
            $message = $message->msg;
        }
        $this->data = new \stdClass();
        foreach ($message as $key => $value) {
            $this->$key = $value;
        }
    }

    public function __toString()
    {
        $message = array();
        $message['ver'] = self::$version;
        $message['msg'] = array(
            'type' => $this->type,
            'src' => $this->src,
            'dst' => $this->dst,
            'data' => $this->data,
        );

        return json_encode($message, JSON_UNESCAPED_UNICODE);
    }

    public function getType()
    {
        return $this->type;
    }

    public function setType($type)
    {
        $this->type = $type;
    }

    public function getSrc()
    {
        return $this->src;
    }

    public function setSrc($src)
    {
        $this->src = $src;
    }

    public function getDst()
    {
        return $this->dst;
    }

    public function setDst($dst)
    {
        $this->dst = $dst;
    }

    public function prepareResponse()
    {
        $temp = $this->dst;
        $this->dst = $this->src;
        $this->src = $temp;
        $this->type = self::TYPE_RESPONSE;
    }

    public function getId()
    {
        if (property_exists($this->data, 'id') && $this->data->id !== null) {
            return $this->data->id;
        }
        $param = $this->getParams();
        if (property_exists($param, 'id') && $param->id !== null) {
            return $param->id;
        }

        return;
    }

    public function setId($id)
    {
        $this->data->id = $id;
    }

    public function getChanges()
    {
        if (property_exists($this->data, 'changes')) {
            return $this->data->changes;
        }

        return;
    }

    public function setChanges($change)
    {
        $changes = array();
        if (property_exists($this->data, 'changes')) {
            $changes = $this->data['changes'];
        }
        $this->data->changes = (object) array_merge((array) $change, (array) $changes);
    }

    public function getRequestId()
    {
        if (property_exists($this->data, 'requestId')) {
            return $this->data->requestId;
        }

        return;
    }

    public function setRequestId($id)
    {
        $this->data->requestId = $id;
    }

    public function getSuccess()
    {
        if (property_exists($this->data, 'success')) {
            return $this->data->success;
        }

        return false;
    }

    public function setSuccess($success)
    {
        $this->data->success = $success;
    }

    public function getError()
    {
        if (property_exists($this->data, 'errors')) {
            return $this->data->errors;
        }

        return;
    }

    public function setError($errors)
    {
        $this->data->success = false;
        $this->data->errors = $errors;
    }

    public function getData()
    {
        return $this->data;
    }

    public function setData($data)
    {
        $this->data = $data;
    }

    public function setCommand($cmd)
    {
        if (!$this->data) {
            $this->data = new \stdClass();
        }
        $this->data->cmd = $cmd;
    }

    public function getCommand()
    {
        if ($this->data && isset($this->data->cmd)) {
            return $this->data->cmd;
        }

        return;
    }

    public function setComponent($component)
    {
        if (!$this->data) {
            $this->data = new \stdClass();
        }
        $this->data->component = $component;
    }

    public function getComponent()
    {
        if ($this->data && isset($this->data->component)) {
            return $this->data->component;
        } elseif ($this->data && !isset($this->data->component) && isset($this->data->param) && isset($this->data->param->components) && is_array($this->data->param->components)) {
            return $this->data->param->components;
        }

        return;
    }

    public function hasComponent($component)
    {
        $components = $this->getComponent();
        if ($components == null) {
            $components = array_keys((array) $this->getParams());
        }
        if (is_array($components)) {
            if (array_search($component, $components) !== false) {
                return true;
            }
        } else {
            if ($components === $component) {
                return true;
            }
        }

        return false;
    }

    public function setParams(\stdClass $params)
    {
        if (!$this->data) {
            $this->data = new \stdClass();
        }
        $this->data->param = $params;
    }

    public function appendParam($key, $value)
    {
        if (!$this->data) {
            $this->data = new \stdClass();
        }
        if (!$this->data->param) {
            $this->data->param = new \stdClass();
        }
        $this->data->param->{$key} = $value;
    }

    public function getParams()
    {
        if ($this->data && isset($this->data->param)) {
            return $this->data->param;
        }

        return (object) array();
    }

    public function getClient()
    {
        if ($this->data && isset($this->data->client)) {
            return $this->data->client;
        }

        return (object) array();
    }

    public function setClient(\stdClass $client)
    {
        if (!$this->data) {
            $this->data = new \stdClass();
        }
        $this->data->client = $client;
    }

    public function validate()
    {
        return true;
        if (!$this->type) {
            Logger::error('Message type not supplied');

            return false;
        }

        if (!$this->data) {
            Logger::error('Message data not supplied');

            return false;
        }

        if (!$this->dst) {
            Logger::error('Message dst not supplied');

            return false;
        }

        if (!in_array($this->type, self::$valid_types)) {
            Logger::error('Message type not valid');

            return false;
        }

        // Validate as per the message type
        if ($this->type == self::TYPE_NOTIFY) {
            if (!isset($this->data->component)) {
                Logger::error('Notify missing data.component');

                return false;
            }

            if (!in_array($this->data->component, self::$valid_components)) {
                Logger::error('Invalid data.component');

                return false;
            }
        } elseif ($this->type == self::TYPE_REQUEST) {
            if (!isset($this->data->cmd)) {
                Logger::error('Request missing data.cmd');

                return false;
            }

            if (!isset($this->data->component)) {
                //Logger::comment('Request missing data.component, checking for data.params.components');
                if (!isset($this->data->param) || !property_exists($this->data->param, 'components') || !is_array($this->data->param->components)) {
                    Logger::error('Request missing data.params.component');

                    return false;
                }
            }

            if (!isset($this->data->requestId)) {
                Logger::error('Request missing data.requestId');

                return false;
            }

            if (isset($this->data->cmd) && !in_array($this->data->cmd, self::$valid_commands)) {
                Logger::error('Invalid data.cmd');

                return false;
            }

            if (isset($this->data->component) && !in_array($this->data->component, self::$valid_components)) {
                Logger::error('Invalid data.component');

                return false;
            }

            if (!isset($this->data->component) && isset($this->data->param) && is_array($this->data->param->components)) {
                $diffArray = array_diff($this->data->param->components, self::$valid_components);
                if (count($diffArray) > 0) {
                    Logger::error('Invalid data.param.components containing unknown components, '.implode(',', $diffArray));

                    return;
                }
            }
        } elseif ($this->type == self::TYPE_RESPONSE) {
            return true;
        } elseif ($this->type == self::TYPE_PING) {
        }

        return true;
    }
}
