<?php
	namespace FutureHome\Socket;

	use Evenement\EventEmitter;
    use FutureHome\Socket\Message\SevenOfNine\Message;
    use Symfony\Component\Console\Output\ConsoleOutput;

    class Site extends EventEmitter{
        const TYPE_DEVICE = 'device';
        const TYPE_USER = 'user';

        const MODE_HOME     = 'home';
        const MODE_WORK     = 'work';
        const MODE_VACATION = 'vacation';
        const MODE_SLEEP    = 'sleep';

        static $loop;

		protected $_identifier;

		protected $_cubes;

		protected $_users;

		protected $_message_queue;

        protected $_site;

        protected $_mode;

        protected $_rogue_cubes;

        protected $_configured;

        protected $_devices;

		/**
		 * [__construct]
		 */
		public function __construct($siteId) {
			$this->_identifier = $siteId;
			$this->_cubes = array();
			$this->_users = array();
			$this->_message_queue = array();

            static::$loop->addPeriodicTimer(10, array($this, 'checkForRogueCubes'));
            static::$loop->addPeriodicTimer(60, array($this, 'heartBeat'));
            $this->on('checkForCubes', array($this, 'checkForRogueCubes'));

            if($this->_identifier) {
                \SiteMap::addSite($this->_identifier, $this);
            }
		}

        public function getMode() {
            return $this->_mode;
        }

        public function setMode($mode) {
            $this->_mode = $mode;
        }

        public function getRoomForDevice($deviceId) {
            if(array_key_exists($deviceId, $this->_devices)) {
                return $this->_devices[$deviceId]->room;
            }
            return null;
        }

        public function setDevices($devices) {
            $this->_devices = array();
            foreach($devices as $device) {
                $this->_devices[$device->id] = $device;
            }
        }

        public function getSiteIdentifier() {
            return $this->_identifier;
        }

		public function setSiteIdentifier($siteId) {
			$this->_identifier = $siteId;
		}

        public function heartBeat() {
            $message = new Message((object)array());

            $message->setSrc(Message::SRC_HIVE);
            $message->setType(Message::TYPE_PING);
            $message->setDst(Message::DST_CLIENT);
            $message->setData((object)array(
                'siteId'    =>  $this->_identifier
            ));

            $this->sendToUsers($message);
            $message->setDst(Message::DST_HYPERCHANNEL);
            $this->sendToDevice($message);
        }

		/**
		 * [attachDevice associate a cube with the site]
		 * @param  [type] $connection [description]
		 * @return [type]             [description]
		 */
		public function attachDevice($connection) {
			$this->_cubes[md5(spl_object_hash($connection))] = new Device($connection);

			if(count($this->_message_queue) > 0) {
				foreach($this->_message_queue as $message) {
					$this->sendToDevice($message);
				}
				$this->_message_queue = array();
			}
            $this->info("Cube {$connection->resourceId} successfully assigned to site {$this->_identifier}");

            $message = new Message((object)array());

            $message->setSrc(Message::SRC_HIVE);
            $message->setDst(Message::DST_CLIENT);
            $message->setType(Message::TYPE_NOTIFY);

            $message->setCommand(Message::COMMAND_SET);
            $message->setComponent(Message::COMPONENT_CUBE);

            $message->setParams(
                (object)array(
                    'message' => 'cube-online'
                )
            );

            $this->sendToUsers($message);

            \ConnectionMap::addConnection($this->_identifier, $connection);

            // Send a house request to the cube
            $houseMessage = new Message((object)array());
            $houseMessage->setSrc(Message::SRC_HIVE);
            $houseMessage->setDst(Message::DST_VINCULUM);
            $houseMessage->setType(Message::TYPE_REQUEST);

            $houseMessage->setData((object)array(
                'requestId' => time(),
                'component' => null,
                'cmd'       => Message::COMMAND_GET,
                'param'    => (object)array(
                    'components' => array(Message::COMPONENT_HOUSE)
                )
            ));

            $this->sendToDevice($houseMessage);

		}

        public function getUser($connection) {
            return $this->_users[$connection];
        }

		/**
		 * [addUser description]
		 * @param [string] $name       [Name of the user]
		 * @param [string] $country    [Country of the user]
		 * @param [string] $connection [Incoming socket connection]
		 */
		public function addUser($connection) {
			$user = new User($connection);
			$this->_users[md5(spl_object_hash($connection))] = $user;

            $this->info("User successfully assigned to site {$this->_identifier}");

            \ConnectionMap::addConnection($this->_identifier, $connection);
		}

		/**
		 * [addConnection description]
		 * @param Array  $session    [The session data]
		 * @param $connection [The connection interface]
		 * @throws [Exception] If invalid session information is sent
		 */
		public function addConnection(Array $session, $connection) {
			/**
			 * Note that the extra type of users such as demo/public etc will be handled by Public Control Class
			 */
			if($session['type'] == self::TYPE_DEVICE) {
				$this->attachDevice($connection);
			}
			else {
				$this->addUser($connection);
			}
		}

		/**
		 * [sendToDevice sends message to device]
		 * @param  Array  $message
		 */
		public function sendToDevice(Message $message) {
			if(count($this->_cubes) > 0) {
                foreach($this->_cubes as $cube)
				    $cube->send($message);
			}
			else {
                if($this->_configured === true) {
                    $m = new Message((object)array());

                    $m->setSrc(Message::SRC_HIVE);
                    $m->setDst(Message::DST_CLIENT);
                    $m->setType(Message::TYPE_NOTIFY);

                    $m->setCommand(Message::COMMAND_SET);
                    $m->setComponent(Message::COMPONENT_CUBE);

                    $m->setParams(
                        (object)array(
                            'message' => 'cube-offline'
                        )
                    );

                    $this->sendToUsers($m);
                }
                else {
                    $m = new Message((object)array());

                    $m->setSrc(Message::SRC_HIVE);
                    $m->setDst(Message::DST_CLIENT);
                    $m->setType(Message::TYPE_NOTIFY);

                    $m->setCommand(Message::COMMAND_SET);
                    $m->setComponent(Message::COMPONENT_CUBE);

                    $m->setParams(
                        (object)array(
                            'message' => 'no-cube'
                        )
                    );

                    $this->sendToUsers($m);
                }
                $this->emit('checkForCubes');
				if(array_search($message, $this->_message_queue) === FALSE) {
					$this->_message_queue[] = $message;
				}
                return false;
			}
            return true;
		}

		/**
		 * [sendToUsers sends message to all connected users]
		 * @param  Array  $message
		 * @return [type]
		 */
		public function sendToUsers(Message $message)
        {
            $destination = $message->getDst();
            if ($destination && isset($this->_users[$destination]) && $destination !== Message::DST_CLIENT) {
                $this->info("Sending message to " . $destination);
                if($this->guessConnectionType($message->getSrc()) == self::TYPE_DEVICE) {
                    $message->setSrc(Message::SRC_HYPERCHANNEL);
                }
                $message->setDst(Message::DST_CLIENT);
                $user = $this->_users[$destination];
                $user->send($message);
            } else if($destination === Message::DST_CLIENT && $message->getType() == Message::TYPE_NOTIFY) {
                $this->info("Broadcasting the message to all users");
                if($this->guessConnectionType($message->getSrc()) == self::TYPE_DEVICE) {
                    $message->setSrc(Message::SRC_HYPERCHANNEL);
                }
                $message->setDst(Message::DST_CLIENT);
                foreach ($this->_users as $connection => $user) {
                    $user->send($message);
                }
            }
            else if($destination === Message::DST_CLIENT && $message->getType() == Message::TYPE_PING) {
                $this->info("Sending heartbeat to all users");
                foreach ($this->_users as $connection => $user) {
                    $user->send($message);
                }
            }
            else {
                $this->error("Couldn't figure out where to send the message $message to.");
            }
        }

		/**
		 * [handleMessage handle Site related message]
		 * @param  mixed  $message    [message received from device/user]
		 * @param  String $connection [description]
		 */
		public function handleMessage( $message, $connection ) {
            // Force override source to prevent client directing responses to other clients
            $message->setSrc($connection);

            if ($this->guessConnectionType($connection) === self::TYPE_DEVICE) {
                $this->sendToUsers($message);
            }
            else {
                $this->sendToDevice($message);
            }
		}

		/**
		 * [handleDisconnect Handles disconnection of a connection related to the site]
		 * @param  String $connection [description]
		 */
		public function handleDisconnect($connection) {
            if ($this->guessConnectionType($connection) === self::TYPE_DEVICE) {
                $cube = $this->_cubes[$connection];
                unset($cube);
				unset($this->_cubes[$connection]);

                $message = new Message((object)array());

                $message->setSrc(Message::SRC_HIVE);
                $message->setDst(Message::DST_CLIENT);
                $message->setType(Message::TYPE_NOTIFY);

                $message->setCommand(Message::COMMAND_SET);
                $message->setComponent(Message::COMPONENT_CUBE);

                $message->setParams(
                    (object)array(
                        'message' => 'cube-offline'
                    )
                );

                $this->sendToUsers($message);
			}
			else {
				// Run, It's a user
				$user = $this->_users[$connection];
				unset($user);
				unset($this->_users[$connection]);
			}
		}

        public function checkForRogueCubes()
        {
            // $userCount = count($this->_users);
            // $this->info("Checking for rogue cube on {$this->_identifier} with {$userCount} users");
            // if($this->_identifier && $userCount > 0) {
            //     foreach($this->_users as $connection => $user) {
            //         $this->emit('rogueCubeCheck', array($user->getIP(), $connection));
            //     }
            // }
        }

        public function notifyRogue($connection, $cubes) {
            $this->_rogue_cubes = $cubes;

            if(count($this->_rogue_cubes) > 0) {
                $message = new Message((object) array(
                    'data' => (object)array(
                        'component' => Message::COMPONENT_CUBE,
                        'cmd'   =>      Message::COMMAND_SET,
                        'param' => (object)array(
                            'message' => 'rogue',
                            'count' => count($cubes)
                        )
                    )
                ));

                $message->setSrc(Message::SRC_HIVE);
                $message->setDst($connection);
                $message->setType(Message::TYPE_NOTIFY);

                $this->sendToUsers($message);
            }
            else {
                Logger::info("No rogue cubes found");
            }
        }

        public function guessConnectionType($connection) {
            foreach($this->_cubes as $cube) {
                if ($connection === $cube->getHash()) {
                    return self::TYPE_DEVICE;
                }
            }

            return self::TYPE_USER;
        }

        /**
         * Write a string as information output.
         *
         * @param  string  $string
         * @return void
         */
        protected function info($string)
        {
            Logger::info($string);
        }

        /**
         * Write a string as standard output.
         *
         * @param  string  $string
         * @return void
         */
        protected function line($string)
        {
            Logger::line($string);
        }

        /**
         * Write a string as comment output.
         *
         * @param  string  $string
         * @return void
         */
        protected function comment($string)
        {
            Logger::comment($string);
        }

        /**
         * Write a string as error output.
         *
         * @param  string  $string
         * @return void
         */
        protected function error($string)
        {
            Logger::error($string);
        }
	}