'use strict';

module.exports = function(grunt){

  require('matchdep').filterDev('grunt-*').forEach(grunt.loadNpmTasks);

  grunt.initConfig({
    pkg: grunt.file.readJSON('package.json'),

    watch: {
      css: {
        files: ['public/css/**/*.scss'],
        tasks: ['buildcss']
      }
    },

    sass: {
      build: {
        files: {
          'public/css/app.css': 'public/css/app.scss'
        }
      }
    },

    csslint: {
      strict: {
        src: 'public/css/app.css'
      }
    },

    cssmin: {
      build: {
        src: 'public/css/app.css',
        dest: 'public/css/app.min.css'
      }
    },

    copy: {
      loader: {
        src: 'app/config/app.prod.php',
        dest: 'app/config/app.php'
      },
      views: {
        src: 'app/views/app.prod.php',
        dest: 'app/views/app.php'
      },
      propel: {
        src: 'app/config/propel/config-prod.php',
        dest: 'app/config/propel/config.php'
      },
      config: {
        src: 'app/config/database-prod.php',
        dest: 'app/config/database.php'
      },
      dev: {
        src: 'app/views/app.dev.php',
        dest: 'app/views/app.php'
      },
    },

    concat: {
      options: {
        separator : ';',
      },
      vendor: {
        src: [
        'public/js/vendor/zepto/zepto.js',
        'public/js/vendor/angular/angular.js',
        'public/js/vendor/angular-route/angular-route.js',
        'public/js/vendor/angular-ui-utils/ui-utils.js',
        'public/js/vendor/angular-bootstrap/ui-bootstrap-tpls.js',
        'public/js/vendor/fastclick/lib/fastclick.js',
        // 'public/js/vendor/fabric/fabric.js',
        'public/js/vendor/nouislider/jquery.nouislider.min.js',
        // 'public/js/vendor/cordova/cordova.js',
        // 'public/js/vendor/cordova/index.js',
        ],
        dest: 'public/js/app/.temp/vendor.js',
      },
      app: {
        src: 'public/js/app/app.js',
        dest: 'public/js/app/.temp/app.js'
      },
      services: {
        src: 'public/js/app/services/**/*.js',
        dest: 'public/js/app/.temp/services.js'
      },
      controllers: {
        src: 'public/js/app/controllers/**/*.js',
        dest: 'public/js/app/.temp/controllers.js'
      },
      filters: {
        src: 'public/js/app/filters/**/*.js',
        dest: 'public/js/app/.temp/filters.js'
      },
      directives: {
        src: 'public/js/app/directives/**/*.js',
        dest: 'public/js/app/.temp/directives.js'
      },
      final: {
        src: [
        'public/js/app/.temp/vendor.js',
        'public/js/app/.temp/app.js',
        'public/js/app/.temp/services.js',
        'public/js/app/.temp/controllers.js',
        'public/js/app/.temp/filters.js',
        'public/js/app/.temp/directives.js'
        ],
        dest: 'public/js/app/app.min.js'
      }
    },

    uglify: {
      options: {
        mangle : false
      },
      build: {
        files: {
          'public/js/app/.temp/vendor.js'        : 'public/js/app/.temp/vendor.js',
          // 'public/js/app/.temp/app.js'        : 'public/js/app/.temp/app.js',
          'public/js/app/.temp/services.js'      : 'public/js/app/.temp/services.js',
          'public/js/app/.temp/controllers.js'   : 'public/js/app/.temp/controllers.js',
          'public/js/app/.temp/filters.js'       : 'public/js/app/.temp/filters.js',
          // 'public/js/app/.temp/directives.js' : 'public/js/app/.temp/directives.js',
        }
      }
    },

    clean: {
      js: 'public/js/app/.temp',
      css: '.sass-cache'
    }
  });

grunt.registerTask('buildcss', ['sass', 'cssmin', 'clean:css']);

grunt.registerTask('buildjs',  [
  'concat:vendor',
  'concat:app',
  'concat:services',
  'concat:controllers',
  'concat:filters',
  'concat:directives',
  'uglify',
  'concat:final'
  ]);

grunt.registerTask('build', ['buildcss', 'buildjs', 'copy:loader', 'copy:views', 'copy:propel', 'copy:config', 'clean']);
grunt.registerTask('dev', ['buildcss', 'buildjs', 'copy:dev', 'clean']);

};
